#!/usr/local/bin/python
"""\
--------------------------------------------------------------------
 fastaGrep.py
--------------------------------------------------------------------
fastaGrep.py [option] <argument>
--------------------------------------------------------------------
-h    --help                       : print this help

-s    --sequence=<pattern>         : match the sequence with 
                                     a regular pattern
                                     
-i    --identifier=<pattern>       : match the sequence 
                                     identifier with a regular 
                                     pattern
                                     
-d    --definition=<pattern>       : match the sequence definition 
									 with a regular pattern
									 
-a    --attribute=<name>:<pattern> : match the sequence attribute 
									 <name> with a regular pattern
									 
-l    --lmin=##                    : keep sequences longer than 
									 lmin

-L    --lmax=##                    : keep sequences shorter than 
									 lmax

-v                                 : revert the sequence selection
--------------------------------------------------------------------
"""

import fileinput
import re
import getopt
import sys

from obitools.fasta import fastaIterator,writeFasta
from obitools.utils import checkHelpOption

	
def goodFastaGenerator():
    o,filenames = getopt.getopt(sys.argv[1:],
                                'hs:i:d:a:l:L:v',
                                ['help',
                                 'sequence=',
                                 'identifier=',
                                 'definition=',
                                 'attribute=',
                                 'lmin=',
                                 'lmax='])
                                 
    sys.argv[1:]=filenames
                                 
    haveSequencePattern  = False
    haveIdentifierPattern= False
    haveDefinitionPattern= False
    haveAttributePattern = False
    haveLmin             = False
    haveLmax             = False
    
    isInverted           = False
    
    attributePatterns={}
    
    for name,value in o:
        if name in ('-s','--sequence'):
            sequencePattern=re.compile(value,re.Ignore)
            haveSequencePattern=True
            
        elif name in ('-d','--definition'):
            definitionPattern=re.compile(value)
            haveDefinitionPattern=True
            
        elif name in ('-i','--identifier'):
            identifierPattern=re.compile(value)
            haveIdentifierPattern=True

        elif name in ('-a','--attribute'):
            attribute,pattern=value.split(':',1)
            attributePatterns[attribute]=re.compile(pattern)
            haveAttributePattern=True
            
        elif name in ('-l','--lmin'):
            lmin=int(value)
            haveLmin=True

        elif name in ('-L','--lmax'):
            lmax=int(value)
            haveLmax=True

        elif name in ('-v'):
            isInverted=True
            
        else:
            raise ValueError,'Unknown option %s' % name
            
    def sequenceSelector(seq):
    
        good=True
        
        if haveSequencePattern:
            good = bool(sequencePattern.search(str(seq)))
            
        if good and haveIdentifierPattern:
            good = bool(identifierPattern.search(seq.id))
    
        if good and haveDefinitionPattern:
            good = bool(definitionPattern.search(seq.definition))
    
        if good and haveAttributePattern:
            good = (reduce(lambda x,y : x and y,
                           (bool(attributePatterns[p].search(seq[p]))
                            for p in attributePatterns
                             if p in seq),True)
                    and
                    reduce(lambda x,y : x and y,
                           (bool(p in seq)
                            for p in attributePatterns),True)
                   )
                   
        if good and haveLmin:
            good = len(seq) >= lmin
            
        if good and haveLmax:
            good = len(seq) <= lmax
                   
                            
        if isInverted:
            good=not good
                       
        return good
    
    return sequenceSelector
    
        
	
if __name__=='__main__':

    checkHelpOption(__doc__)

    goodFasta=goodFastaGenerator()
    
    fasta = fastaIterator(fileinput.input())
    
    for seq in fasta:
        if goodFasta(seq):
            print writeFasta(seq)
            
            