#!/usr/local/bin/python


import fileinput
import re
import getopt
import sys

from obitools.fasta import fastaIterator,writeFasta

	
def annoteFastaGenerator():
    o,filenames = getopt.getopt(sys.argv[1:],
                                'hs:i:d:a:l:L:t:v',
                                ['help',
                                 'sequence=',
                                 'identifier=',
                                 'definition=',
                                 'attribute=',
                                 'lmin=',
                                 'lmax=',
                                 'tag='])
                                 
    sys.argv[1:]=filenames
                                 
    haveSequencePattern  = False
    haveIdentifierPattern= False
    haveDefinitionPattern= False
    haveAttributePattern = False
    haveLmin             = False
    haveLmax             = False
    haveTag              = False
    
    isInverted           = False
    
    attributePatterns={}
    tags={}
    
    for name,value in o:
        if name in ('-h','--help'):
            printHelp()
            exit()
        elif name in ('-s','--sequence'):
            sequencePattern=re.compile(value,re.Ignore)
            haveSequencePattern=True
            
        elif name in ('-d','--definition'):
            definitionPattern=re.compile(value)
            haveDefinitionPattern=True
            
        elif name in ('-i','--identifier'):
            identifierPattern=re.compile(value)
            haveIdentifierPattern=True

        elif name in ('-a','--attribute'):
            attribute,pattern=value.split(':',1)
            attributePatterns[attribute]=re.compile(pattern)
            haveAttributePattern=True
            
        elif name in ('-t','--tag'):
            attribute,data=value.split(':',1)
            tags[attribute]=data.strip()
            haveTag=True
            
        elif name in ('-l','--lmin'):
            lmin=int(value)
            haveLmin=True

        elif name in ('-L','--lmax'):
            lmax=int(value)
            haveLmax=True

        elif name in ('-v'):
            isInverted=True
            
        else:
            raise ValueError,'Unknown option %s' % name
            
            
    def sequenceAnnotator(seq):
    
        good=True
        
        if haveSequencePattern:
            good = bool(sequencePattern.search(str(seq)))
            
        if good and haveIdentifierPattern:
            good = bool(identifierPattern.search(seq.id))
    
        if good and haveDefinitionPattern:
            good = bool(definitionPattern.search(seqdefinition))
    
        if good and haveAttributePattern:
            good = (reduce(lambda x,y : x and y,
                           (bool(attributePatterns[p].search(seq[p]))
                            for p in attributePatterns
                             if p in seq),True)
                    and
                    reduce(lambda x,y : x and y,
                           (bool(p in seq)
                            for p in attributePatterns),True)
                   )
                   
        if good and haveLmin:
            good = len(seq) >= lmin
            
        if good and haveLmax:
            good = len(seq) <= lmax
                   
                            
        if isInverted:
            good=not good
            
        if good:
            info.update(tags)
                       
        return info
        
    assert haveTag,'You must specified at least one --tag option'
    
    return sequenceAnnotator
    
    
def printHelp():
    print "-----------------------------------"
    print " fastaTag.py"
    print "-----------------------------------"
    print "fastaGrep.py [option] <argument>"
    print "-----------------------------------"
    print "-h    --help                       : print this help"
    print "-s    --sequence=<pattern>         : match the sequence with a regular pattern"
    print "-i    --identifier=<pattern>       : match the sequence identifier with a regular pattern"
    print "-d    --definition=<pattern>       : match the sequence definition with a regular pattern"
    print "-a    --attribute=<name>:<pattern> : match the sequence attribute <name> with a regular pattern"
    print "-l    --lmin=##                    : keep sequences longer than lmin"
    print "-L    --lmax=##                    : keep sequences shorter than lmax"
    print "-v                                 : revert the sequence selection"
    print "-----------------------------------"
    
	
if __name__=='__main__':

    annoteFasta=annoteFastaGenerator()
    
    fasta = fastaIterator(fileinput.input())
    
    for seq in fasta:
        info=annoteFasta(seq)          
        print writeFasta(seq)
            
            